/*
 * Copyright (C) 2008-2010 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.greaze.rest.server;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.logging.Logger;

import javax.servlet.http.HttpServletResponse;

import com.google.greaze.definition.ContentBodySpec;
import com.google.greaze.definition.LogConfig;
import com.google.greaze.definition.rest.ResourceId;
import com.google.greaze.definition.rest.RestResourceBase;
import com.google.greaze.definition.rest.RestResponseBase;
import com.google.greaze.definition.webservice.WebServiceResponse;
import com.google.greaze.webservice.server.ResponseSender;
import com.google.gson.Gson;

/**
 * Sends a JSON web service response on {@link HttpServletResponse}.
 * 
 * @author Inderjeet Singh
 */
public class RestResponseSender<I extends ResourceId, R extends RestResourceBase<I, R>>
    extends ResponseSender {
  private static final Logger logger = Logger.getLogger(RestResponseSender.class.getCanonicalName());

  public RestResponseSender(Gson gson) {
    super(gson);
  }

  @SuppressWarnings("unchecked")
  @Override
  public void send(HttpServletResponse conn, WebServiceResponse response) {
    if (response instanceof RestResponseBase) {
      send(conn, (RestResponseBase<I, R>)response);
    } else {
      super.send(conn, response);
    }
  }

  public void send(HttpServletResponse res, RestResponseBase<I, R> response) {
    try {
      sendHeaders(res, response.getHeaders());
      sendBody(res, response.getResource(), response.getBody().getSpec().getBodyJavaType());
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
 
  // We could reuse the base classes method, sendBody. However, that requires that
  // a ResponseBody.GsonTypeAdapter is registered. We avoid that registration for REST resources
  // with this implementation
  private void sendBody(HttpServletResponse res, R responseBody, Type responseBodyType)
      throws IOException {
    res.setContentType(ContentBodySpec.JSON_CONTENT_TYPE);
    res.setCharacterEncoding(ContentBodySpec.JSON_CHARACTER_ENCODING);
    String json = gson.toJson(responseBody, responseBodyType);
    if (LogConfig.INFO) logger.info("Response Body:" + json);
    res.getWriter().append(json);
  }
}
